import string
import random


def generate_username(size=8, chars=string.digits + string.ascii_lowercase) -> str:
    """
    Generates random username for CPanels
    :param size: username length
    :param chars: characters to use during generation
    :return: str: generated username
    """
    return ''.join(random.choice(chars) for _ in range(size))


def generate_password(size=14, chars=string.ascii_letters + string.ascii_uppercase + string.digits) -> str:
    """
    Generates random password for CPanels
    :param size: password length
    :param chars: characters to use during generation
    :return: str: generated password
    """
    return ''.join(random.choice(chars) for _ in range(size)) + '@' + str(random.randint(1, 999))



API_COMMANDS = [
    '_getpkgextensionform',
    'abort_transfer_session',
    'accesshash',
    'accountsummary',
    'acctcounts',
    'activate_zone_key',
    'add_override_features_for_user',
    'add_zone_key',
    'adddns',
    'addpkg',
    'addpkgext',
    'addzonerecord',
    'api_token_create',
    'api_token_get_details',
    'api_token_list',
    'api_token_revoke',
    'api_token_update',
    'applist',
    'batch',
    'changepackage',
    'convert_addon_fetch_conversion_details',
    'convert_addon_fetch_domain_details',
    'convert_addon_get_conversion_status',
    'convert_addon_initiate_conversion',
    'convert_addon_list_addon_domains',
    'convert_addon_list_conversions',
    'cors_proxy_get',
    'create_featurelist',
    'create_integration_group',
    'create_integration_link',
    'create_parked_domain_for_user',
    'create_user_session',
    'createacct',
    'deactivate_zone_key',
    'delete_customizations',
    'delete_featurelist',
    'delete_ssl_vhost',
    'delpkgext',
    'disable_dnssec_for_domains',
    'disable_mail_sni',
    'domainuserdata',
    'dumpzone',
    'editpkg',
    'editquota',
    'editzonerecord',
    'emailtrack_search',
    'emailtrack_stats',
    'emailtrack_user_stats',
    'enable_dnssec_for_domains',
    'enable_mail_sni',
    'enqueue_deferred_ssl_installations',
    'enqueue_transfer_item',
    'export_zone_dnskey',
    'export_zone_files',
    'export_zone_key',
    'fetch_ds_records_for_domains',
    'fetch_ssl_certificates_for_fqdns',
    'fetch_ssl_vhosts',
    'fetch_transfer_session_log',
    'fetch_vhost_ssl_components',
    'fetchcrtinfo',
    'fetchsslinfo',
    'forcepasswordchange',
    'generate_mobileconfig',
    'generatessl',
    'get_available_applications',
    'get_available_authentication_providers',
    'get_available_featurelists',
    'get_best_ssldomain_for_service',
    'get_current_users_count',
    'get_disk_usage',
    'get_domain_info',
    'get_feature_metadata',
    'get_feature_names',
    'get_featurelist_data',
    'get_featurelists',
    'get_integration_link_user_config',
    'get_maximum_users',
    'get_nameserver_config',
    'get_password_strength',
    'get_public_contact',
    'get_public_ip',
    'get_remote_access_hash',
    'get_shared_ip',
    'get_transfer_session_state',
    'get_upgrade_opportunities',
    'get_users_authn_linked_accounts',
    'get_users_links',
    'getdomainowner',
    'gethostname',
    'getpkginfo',
    'getresellerips',
    'getzonerecord',
    'has_digest_auth',
    'has_mycnf_for_cpuser',
    'hold_outgoing_email',
    'import_zone_key',
    'installssl',
    'is_sni_supported',
    'killdns',
    'killpkg',
    'limitbw',
    'link_user_authn_provider',
    'list_integration_groups',
    'list_integration_links',
    'list_mysql_databases_and_users',
    'list_styles',
    'list_users',
    'listaccts',
    'listcrts',
    'listmxs',
    'listpkgs',
    'listsuspended',
    'listzones',
    'load_style',
    'lookupnsip',
    'lookupnsips',
    'mail_sni_status',
    'mass_edit_dns_zone',
    'massmodifyacct',
    'matchpkgs',
    'modifyacct',
    'myprivs',
    'nvget',
    'nvset',
    'parse_dns_zone',
    'passwd',
    'pause_transfer_session',
    'read_featurelist',
    'rebuild_mail_sni_config',
    'rebuilduserssldb',
    'release_outgoing_email',
    'remove_integration_group',
    'remove_integration_link',
    'remove_logo',
    'remove_override_features_for_user',
    'remove_style',
    'remove_zone_key',
    'removeacct',
    'removezonerecord',
    'resetzone',
    'resolvedomainname',
    'retrieve_customizations',
    'save_style',
    'savemxs',
    'set_default',
    'set_digest_auth',
    'set_nsec3_for_domains',
    'set_primary_servername',
    'set_public_contact',
    'setresellernameservers',
    'showbw',
    'start_transfer_session',
    'suspend_outgoing_email',
    'suspendacct',
    'systemloadavg',
    'transfer_module_schema',
    'twofactorauth_generate_tfa_config',
    'twofactorauth_get_issuer',
    'twofactorauth_get_user_configs',
    'twofactorauth_policy_status',
    'twofactorauth_remove_user_config',
    'twofactorauth_set_issuer',
    'twofactorauth_set_tfa_config',
    'uapi_cpanel',
    'unlink_user_authn_provider',
    'unset_nsec3_for_domains',
    'unsuspend_outgoing_email',
    'unsuspendacct',
    'update_customizations',
    'update_featurelist',
    'update_integration_link_token',
    'verify_new_username',
    'verify_new_username_for_restore',
    'verify_user_has_feature',
    'version'
]


import setuptools

setuptools.setup(
   name='WhamBamPy',
   version='0.0.7',
   author='Dmitry Minin',
   author_email='dmitry.m@bbdmgroup.com',
   packages=['whambampy'],
   scripts=[],
   url='http://pypi.python.org/pypi/whambampy/',
   license='LICENSE.txt',
   description='WHM Api interface',
   long_description=open('README.md').read(),
   long_description_content_type='text/markdown',
   install_requires=[
      "certifi==2022.9.24",
      "charset-normalizer==2.1.1",
      "colorama==0.4.5",
      "idna==3.4",
      "requests==2.28.1",
      "urllib3==1.26.12"
   ],
   download_url="https://github.com/BBDM-Group/bbdm-whm-manager/archive/refs/tags/0.0.7.tar.gz"
)


import logging
import requests
import urllib3

from typing import Any

from whambampy.api_commands import API_COMMANDS
from whambampy.defaults import *

from urllib3.exceptions import InsecureRequestWarning


urllib3.disable_warnings(InsecureRequestWarning)
logger = logging.getLogger(__name__)


class WHM:
    def __init__(self, host, username, api_token, *args, **kwargs):
        self.host = host
        self.username = username
        self.api_token = api_token

        self._api_version = kwargs.get('api_version', API_VERSION)
        self._whm_port = kwargs.get('whm_port', WHM_PORT)
        self._whm_protocol = kwargs.get('whm_protocol', WHM_PROTOCOL)
        self._api_type = kwargs.get('api_type', API_TYPE)
        self._verify_ssl = kwargs.get('verify_ssl', VERIFY_SSL)
        self._headers = self._get_auth_header()
        self._sec_token = self._get_sec_token()

    def _get_auth_header(self) -> dict:
        return {
            "Authorization": f"whm {self.username}:{self.api_token}",
        }

    def _get_sec_token(self) -> str:
        auth_url = f"{self._whm_protocol}{self.host}:{self._whm_port}/{self._api_type}/create_user_session?" \
            f"api.version={self._api_version}&user={self.username}&service=whostmgrd"
        auth_resp = requests.get(
            url=auth_url,
            headers=self._headers,
            verify=self._verify_ssl,
        )
        if auth_resp.status_code != 200:
            raise PermissionError('Unable to login with the provided credentials')
        return auth_resp.json()['data'].get('cp_security_token')

    def _validate_function(self, function: str) -> bool:
        return function in API_COMMANDS

    def call(self, function: str, **params) -> Any:
        if self._validate_function(function):
            params['api.version'] = self._api_version
            url = f"{self._whm_protocol}{self.host}:{self._whm_port}{self._sec_token}/{self._api_type}/{function}"
            return requests.get(
                url=url,
                headers=self._headers,
                params=params,
                verify=self._verify_ssl,
            ).json()
        else:
            raise TypeError('Called function is not the list of available functions')


def whm(host, username, api_token, *args, **kwargs):
    return WHM(host, username, api_token, *args, **kwargs)
